@@ -21,8 +21,8 @@
 #ifndef __ACTIONS_H__
 #define __ACTIONS_H__
 
-#include <gphoto2-camera.h>
-#include <gphoto2-context.h>
+#include <gphoto2/gphoto2-camera.h>
+#include <gphoto2/gphoto2-context.h>
 
 #include <gp-params.h>
 
@@ -71,9 +71,18 @@ int set_filename_action    (GPParams *, const char *filename);
 int print_version_action   (GPParams *);
 int override_usbids_action (GPParams *, int usb_vendor, int usb_product, 
 			    int usb_vendor_modified, int usb_product_modified);
-int debug_action           (GPParams *);
+int debug_action           (GPParams *, const char *debug_logfile_name);
 int list_config_action     (GPParams *);
 int get_config_action      (GPParams *, const char *name);
 int set_config_action      (GPParams *, const char *name, const char *value);
 
+void _get_portinfo_list	(GPParams *p);
 #endif /* __ACTIONS_H__ */
+
+
+/*
+ * Local Variables:
+ * c-file-style:"linux"
+ * indent-tabs-mode:t
+ * End:
+ */
