@@ -46,6 +46,12 @@
 # define MIN(a, b) ((a) < (b) ? (a) : (b))
 #endif
 
+#ifdef __GNUC__
+#define __unused__ __attribute__((unused))
+#else
+#define __unused__
+#endif
+
 #define CHECK(result) {int r=(result);if(r<0) return(r);}
 #define CL(result,list) {int r=(result);if(r<0) {gp_list_free(list);return(r);}}
 #define CHECK_CONT(result)					\
@@ -88,6 +94,7 @@ static int shell_get_config    (Camera *, const char *);
 static int shell_set_config    (Camera *, const char *);
 static int shell_capture_image (Camera *, const char *);
 static int shell_mkdir         (Camera *, const char *);
+static int shell_rmdir         (Camera *, const char *);
 
 #define MAX_FOLDER_LEN 1024
 #define MAX_FILE_LEN 1024
@@ -120,6 +127,7 @@ struct _ShellFunctionTable {
 	 N_("[directory/]filename"), 1},
 	{"delete", shell_del, N_("Delete"), N_("[directory/]filename"), 1},
 	{"mkdir", shell_mkdir, N_("Create Directory"), N_("directory"), 1},
+	{"rmdir", shell_rmdir, N_("Remove Directory"), N_("directory"), 1},
 #ifdef HAVE_LIBEXIF
 	{"show-exif", shell_show_exif, N_("Show EXIF information"),
 	 N_("[directory/]filename"), 1},
@@ -142,10 +150,12 @@ struct _ShellFunctionTable {
 #define SHELL_PROMPT "gphoto2: {%s} %s> "
 int 	shell_done		= 0;
 
-static int
+static unsigned int
 shell_arg_count (const char *args)
 {
-	int x=0, in_arg=0, count=0;
+	size_t x=0;
+	int in_arg=0;
+	unsigned int count=0;
 	
 	while (x < strlen (args)) {
 		if ((!isspace((int)(args[x]))) && (!in_arg)) {
@@ -174,7 +184,7 @@ shell_read_line (void)
 		if (strlen (cwd) > 25) {
 			strncpy (buf, "...", sizeof (buf));
 			strncat (buf, &cwd[strlen (cwd) - 22],
-				 sizeof (buf));
+				 sizeof (buf) - strlen(buf) - 1);
 			snprintf (prompt, sizeof (prompt), SHELL_PROMPT, buf,
 				  p->folder);
 		} else
@@ -202,9 +212,11 @@ shell_read_line (void)
 }
 
 static int
-shell_arg (const char *args, int arg_num, char *arg)
+shell_arg (const char *args, unsigned int arg_num, char *arg)
 {
-	int x=0, y=0, in_arg=0, count=0, copy=0;
+	size_t x=0, y=0;
+	unsigned int count=0;
+	int in_arg=0, copy=0;
 
 	if (arg_num > shell_arg_count(args)-1)
 		return (GP_ERROR);
@@ -508,7 +520,7 @@ shell_construct_path (const char *folder_orig, const char *rel_path,
                         rel_path += MIN (strlen (rel_path), 2);
 			continue;
 		}
-		if (!strncmp (rel_path, "../", 3)) {
+		if (!strncmp (rel_path, "../", 3) || !strcmp (rel_path, "..")) {
                         rel_path += MIN (3, strlen (rel_path));
 
                         /* Go up one folder */
@@ -531,11 +543,11 @@ shell_construct_path (const char *folder_orig, const char *rel_path,
 			 * trailing slash
 			 */
 			if (dest_folder[strlen (dest_folder) - 1] != '/')
-				strncat (dest_folder, "/", MAX_FOLDER_LEN);
+				strncat (dest_folder, "/", MAX_FOLDER_LEN - strlen(dest_folder) - 1);
 		}
                 if (slash) {
                         strncat (dest_folder, rel_path,
-                                 MIN (MAX_FOLDER_LEN, slash - rel_path));
+                                 MIN (MAX_FOLDER_LEN - strlen(dest_folder) - 1, slash - rel_path));
                         rel_path = slash + 1;
                 } else {
 
@@ -544,7 +556,7 @@ shell_construct_path (const char *folder_orig, const char *rel_path,
                                 strncpy (dest_filename, rel_path,
                                          MAX_FILE_LEN);
                         else
-                                strncat (dest_folder, rel_path, MAX_FILE_LEN);
+				strncat (dest_folder, rel_path, MAX_FILE_LEN);
                         break;
                 }
         }
@@ -553,7 +565,7 @@ shell_construct_path (const char *folder_orig, const char *rel_path,
 }
 
 static int
-shell_lcd (Camera *camera, const char *arg)
+shell_lcd (Camera __unused__ *camera, const char *arg)
 {
 	char new_cwd[MAX_FOLDER_LEN];
 	int arg_count = shell_arg_count (arg);
@@ -580,7 +592,7 @@ shell_lcd (Camera *camera, const char *arg)
 }
 
 static int
-shell_cd (Camera *camera, const char *arg)
+shell_cd (Camera __unused__ *camera, const char *arg)
 {
 	char folder[MAX_FOLDER_LEN];
 	CameraList *list;
@@ -615,7 +627,7 @@ shell_cd (Camera *camera, const char *arg)
 }
 
 static int
-shell_ls (Camera *camera, const char *arg)
+shell_ls (Camera __unused__ *camera, const char *arg)
 {
 	CameraList *list;
 	char buf[1024], folder[MAX_FOLDER_LEN];
@@ -672,7 +684,8 @@ shell_ls (Camera *camera, const char *arg)
 }
 
 static int
-shell_file_action (Camera *camera, GPContext *context, const char *folder,
+shell_file_action (Camera __unused__ *camera, GPContext __unused__ *context, 
+		   const char *folder,
 		   const char *args, FileAction action)
 {
 	char arg[1024];
@@ -690,7 +703,7 @@ shell_file_action (Camera *camera, GPContext *context, const char *folder,
 }
 
 static int
-shell_get_thumbnail (Camera *camera, const char *arg)
+shell_get_thumbnail (Camera __unused__ *camera, const char *arg)
 {
 	CHECK (shell_file_action (p->camera, p->context, p->folder, arg,
 				  save_thumbnail_action));
@@ -699,7 +712,7 @@ shell_get_thumbnail (Camera *camera, const char *arg)
 }
 
 static int
-shell_get (Camera *camera, const char *arg)
+shell_get (Camera __unused__ *camera, const char *arg)
 {
 	CHECK (shell_file_action (p->camera, p->context, p->folder, arg,
 				  save_file_action));
@@ -708,7 +721,7 @@ shell_get (Camera *camera, const char *arg)
 }
 
 static int
-shell_get_raw (Camera *camera, const char *arg)
+shell_get_raw (Camera __unused__ *camera, const char *arg)
 {
 	CHECK (shell_file_action (p->camera, p->context, p->folder, arg,
 				  save_raw_action));
@@ -717,7 +730,7 @@ shell_get_raw (Camera *camera, const char *arg)
 }
 
 static int
-shell_del (Camera *camera, const char *arg)
+shell_del (Camera __unused__ *camera, const char *arg)
 {
 	CHECK (shell_file_action (p->camera, p->context, p->folder, arg,
 				  delete_file_action));
@@ -727,7 +740,7 @@ shell_del (Camera *camera, const char *arg)
 
 #ifdef HAVE_LIBEXIF
 static int
-shell_show_exif (Camera *camera, const char *arg)
+shell_show_exif (Camera __unused__ *camera, const char *arg)
 {
 	CHECK (shell_file_action (p->camera, p->context, p->folder, arg,
 				  print_exif_action));
@@ -737,7 +750,7 @@ shell_show_exif (Camera *camera, const char *arg)
 #endif
 
 static int
-shell_show_info (Camera *camera, const char *arg)
+shell_show_info (Camera __unused__ *camera, const char *arg)
 {
 	CHECK (shell_file_action (p->camera, p->context, p->folder, arg,
 				  print_info_action));
@@ -746,7 +759,7 @@ shell_show_info (Camera *camera, const char *arg)
 }
 
 static int
-shell_put (Camera *camera, const char *args) {
+shell_put (Camera __unused__ *camera, const char *args) {
 	char arg[1024];
 	unsigned int x;
 	char dest_folder[MAX_FOLDER_LEN], dest_filename[MAX_FILE_LEN];
@@ -768,13 +781,35 @@ shell_mkdir (Camera *camera, const char *args) {
 }
 
 static int
-shell_list_config (Camera *camera, const char *args) {
+shell_rmdir (Camera *camera, const char *args) {
+	char *xarg;
+	int xlen, ret;
+
+	if (*args == ' ')
+		args++;
+
+	/* remove trailing / */
+	xarg = strdup(args);
+	xlen = strlen(xarg);
+	while (xlen > 1) {
+		if (xarg[xlen-1] != '/')
+			break;
+		xarg[xlen-1] = '\0';
+		xlen--;
+	}
+	ret = gp_camera_folder_remove_dir (camera, p->folder, xarg, p->context);
+	free (xarg);
+	return ret;
+}
+
+static int
+shell_list_config (Camera __unused__ *camera, const char __unused__ *args) {
 	CHECK (list_config_action (p));
 	return (GP_OK);      
 }
 
 static int
-shell_get_config (Camera *camera, const char *args) {
+shell_get_config (Camera __unused__ *camera, const char *args) {
 	char arg[1024];
 	unsigned int x;
 
@@ -786,7 +821,7 @@ shell_get_config (Camera *camera, const char *args) {
 }
 
 static int
-shell_set_config (Camera *camera, const char *args) {
+shell_set_config (Camera __unused__ *camera, const char *args) {
 	char arg[1024];
 	char *s,*x;
 
@@ -807,20 +842,20 @@ shell_set_config (Camera *camera, const char *args) {
 }
 
 static int
-shell_capture_image (Camera *camera, const char *args) {
+shell_capture_image (Camera __unused__ *camera, const char *args) {
 	while (*args==' ') args++;
 	return capture_generic (GP_CAPTURE_IMAGE, *args?args:NULL);
 }
 
 int
-shell_exit (Camera *camera, const char *arg)
+shell_exit (Camera __unused__ *camera, const char __unused__ *arg)
 {
 	shell_done = 1;
 	return (GP_OK);
 }
 
 static int
-shell_help_command (Camera *camera, const char *arg)
+shell_help_command (Camera __unused__ *camera, const char *arg)
 {
 	char arg_cmd[1024];
 	unsigned int x;
@@ -853,7 +888,7 @@ shell_help_command (Camera *camera, const char *arg)
 }
 
 static int
-shell_help (Camera *camera, const char *arg)
+shell_help (Camera __unused__ *camera, const char *arg)
 {
 	unsigned int x;
 	int arg_count = shell_arg_count (arg);
@@ -878,3 +913,11 @@ shell_help (Camera *camera, const char *arg)
 
 	return (GP_OK);
 }
+
+
+/*
+ * Local Variables:
+ * c-file-style:"linux"
+ * indent-tabs-mode:t
+ * End:
+ */
